%%  N2 AF-PADs, fit results & MF-PADs
%
%   Code & data for distribution.
%   19/12/16
%
%   Contents:
%       - alignAng, full ADMs(t) determined from the data
%       - data(channel,H), structure with full set of experimental Blm(t) for X, A, B states, H5 and H7 ionization.
%            
%       - RllfitTestAll(channel), structure with fit results (Blm(t) and matrix elements) for X, A and B states, H5 only.
%            See RllfitTestAll(channel).RllfitTest.BLMoutFull for final Blm(t) traces, as per fig 3 in the manuscript.
%            See RllfitTestAll(channel).RllfitTest.calcRaw.Dlnl for mat. elements, as per tables in Supp. Mat.
%
%   Use code below for basic plots.
%
%   See also *.out files for ePolyScat results, use ePSproc to process (https://github.com/phockett/ePSproc).
%
%%  Load

load('N2_AF_MF_results_consolidated_191216.mat');

%%  Plot extracted ADMs

figure; 
plot(alignAng.t,alignAng.ADM.');

%%  Plot experimental Blm(t) (X channel, H5) and alignment fit results 
%   These are the result of the first step in the bootstrapping procedure,eqn.3 in the manuscript
 
 lineCols=lines(5);
 
 figure('color',[1 1 1]); 
 
 % Plot expt. data
for n=2:size(allfit,1)
   plot(alldata(1,:),alldata(n,:),'color',lineCols(n-1,:));
   hold on;
end

legend('0','2','4','6');

% Plot fit, same colours & in a separate loop to simplify legend
for n=2:size(allfit,1)
   plot(allfit(1,:),allfit(n,:),'--','color',lineCols(n-1,:));
   hold on;
end
 
xlim([tADMfit(1) tADMfit(end)]);
 
title('2 pulse ADM fits, alignment t-axis');
xlabel('t/ps');
ylabel('\beta_L');

grid on;


%%  Results by Blm - full res. calcs
%   This plot full experimental Blm(t) and fit results

% Set fit results to plot, n=1,2,3 for X, A and B channels.
n=2;
RllfitTest=RllfitTestAll(n).RllfitTest;

% Set experimental results
plotInd=1;    % rCalc(2);
dataset=strrep('RllfitTestAll(n).dataFile','_','\_');
state=n;
H=1;


% Temporal axis and shift for FITS - set to match expt. defn for manuscript
tShift2=0.5;    % -0.035;
taxis2=input.p2(:,2)-input.p1(1,2)+tShift2;  % taxis set from laser pulse info, plus additional shift to match calculations

% Temporal axis and shift for EXPT (relative to ADM calcs.)
tShift=0.13;  %-0.1+tShift2;
taxis=input.p2(:,2)+tShift; % +input.p1(1,2)+tShift;

tLims=[taxis(1) taxis(end)];

% Set experimental traces
BLMexpt=[data(state,H).meanBLM(1,:); data(state,H).meanBLMnorm(2:end,:)];

% Explicitly set colors to allow for consistency over all plots
lineCols=lines(4);

figure('color',[1 1 1],'name','Rll fitting tests');

% Plot B00(t)
subplot(3,1,1);
plot(taxis,BLMexpt(1,:).','Color',lineCols(1,:));
% Add uncertainties
hold on;
for m=1:size(BLMexpt,2);
    plot(repmat(taxis(m),2,1), [BLMexpt(1,m)+data(state,H).SDBLM(1,m) BLMexpt(1,m)-data(state,H).SDBLM(1,m)].','color',0.7.*[1 1 1]);
end
hold on;
plot(RllfitTest(plotInd).tCalcFull+taxis2(1),RllfitTest(plotInd).BLMoutFull(1,:).','--','Color',lineCols(1,:));
plot(RllfitTest(plotInd).tpoints+taxis2(1),RllfitTest(plotInd).BLMout(1,:).','x','Color',lineCols(1,:));
title(['N2 expt + fittings, tests ' dataset ', #' num2str(plotInd) ', resNorm ' num2str(RllfitTest(plotInd).resnorm) ', ' date]);
xlabel('t/ps');
ylabel('\beta_{00}');
xlim(tLims);

% Plot BLM(t)
subplot(3,1,2);
for n=1:(size(BLMexpt,1)-1)
    plot(taxis,BLMexpt(n+1,:).','Color',lineCols(n+1,:));
    hold on;
    plot(RllfitTest(plotInd).tCalcFull+taxis2(1),RllfitTest(plotInd).BLMoutFull(n+1,:).','--','Color',lineCols(n+1,:));
end
for n=1:(size(BLMexpt,1)-1)     % Plot points separately to avoid them appearing in legend
    plot(RllfitTest(plotInd).tpoints+taxis2(1),RllfitTest(plotInd).BLMout(n+1,:).','x','Color',lineCols(n+1,:));
end

xlabel('t/ps');
ylabel('\beta_{LM}');
legend('2','fit','4','fit','6','fit');
xlim(tLims);

% Add uncertainties
hold on;
for m=1:size(BLMexpt,2);
    plot(repmat(taxis(m),2,size(BLMexpt,1)-1), [BLMexpt(2:end,m)+data(state,H).SDBLMnorm(1,m) BLMexpt(2:end,m)-data(state,H).SDBLMnorm(1,m)].','color',0.7.*[1 1 1]);
end

% Plot differences
subplot(3,1,3);
plot(RllfitTest(plotInd).tpoints+taxis2(1),(RllfitTest(plotInd).BLMin-RllfitTest(plotInd).BLMout).',':x');
title('Difference, calcs vs. fit');
legend('0','2','4','6');
xlim(tLims);
