function Dlnl=DlnlRecon(Rllin,Dlnl,RllDlnlInd);
%
%  Function to reconstruct full set of symmetrized matrix elements (as per ePS results) from unique values.
%  Useful for fitting.
%
%  19/10/16     Added +/-m state duplication
%  08/10/16     v1

for DlnlInd=1:length(Dlnl)
    RllinInd=(RllDlnlInd==DlnlInd);     % Check if state is explicitly set, and which rows it corresponds to
    
    if sum(RllinInd)    % Case for values set explicitly
        % Dlnl(DlnlInd).dlnl(Dlnl(DlnlInd).lmPairsRowInd2,3:4)=Rllin(RllinInd,:);
        if ~sum(Dlnl(DlnlInd).mRepInd)      % For no m-state pairs, set directly from (l,m) index
            Dlnl(DlnlInd).dlnl(Dlnl(DlnlInd).lmPairsRowInd,3:4)=Rllin(RllinInd,:);
            Dlnl(DlnlInd).dlnl(:,3:4)=Dlnl(DlnlInd).dlnl(Dlnl(DlnlInd).lmPairsRowInd2,3:4);
        else
            mRows=logical(Dlnl(DlnlInd).mRepInd);    % For case with identical m-state pairs, set from (l) index only
            Dlnl(DlnlInd).dlnl(Dlnl(DlnlInd).lmPairsRowInd(mRows),3:4)=Rllin(RllinInd,:);
            Dlnl(DlnlInd).dlnl(Dlnl(DlnlInd).lmPairsRowInd(~mRows),3:4)=Rllin(RllinInd,:);
            Dlnl(DlnlInd).dlnl(:,3:4)=Dlnl(DlnlInd).dlnl(Dlnl(DlnlInd).lmPairsRowInd2,3:4);
        end
        
    elseif (Dlnl(DlnlInd).q==1)&&(sum(Dlnl(DlnlInd).repInd))        % Case where q=1 values are set from q=-1 values
        if (Dlnl(DlnlInd-1).q==-1)&&(strcmp(Dlnl(DlnlInd-1).irep,Dlnl(DlnlInd).irep))
            Dlnl(DlnlInd).dlnl(:,3:4)=Dlnl(DlnlInd-1).dlnl(:,3:4);
        end
        
    else
        Dlnl(DlnlInd).dlnl(:,3:4)=NaN;      % Set other cases to NaNs for error checking
        
    end
    
    % Apply rotations/phase corrections - use complex form for simplicity  WORKING, but ugly!
    % rowInd=1:Dlnl(DlnlInd).h;
    Dlnl(DlnlInd).dlnl(:,5)=Dlnl(DlnlInd).dlnl(:,3).*exp(1i.*Dlnl(DlnlInd).dlnl(:,4));  % Set complex form
    rotInd=Dlnl(DlnlInd).degenInd(find(Dlnl(DlnlInd).rotInd));  % Select all states for which rotInd=1
    repInd=(Dlnl(DlnlInd).repInd);
    Rlltemp=Dlnl(DlnlInd).dlnl(rotInd,5);   
    if sum(rotInd)  % Case for original (q=-1) set
        Dlnl(DlnlInd).dlnl(rotInd,5)=-imag(Rlltemp)+1i.*real(Rlltemp);    % Switch elements, case for q=-1
    end
    % if sum(rotInd) & (sum(repInd)>0)
%     if sum(repInd)>0
%         Dlnl(DlnlInd).dlnl(rotInd,5)=-real(Rlltemp)-1i.*imag(Rlltemp);    % Switch elements, for q=1 after prior q=-1 switch! 
%     end
    if sum(Dlnl(DlnlInd).conjInd)
        Dlnl(DlnlInd).dlnl(Dlnl(DlnlInd).conjInd,5)=-real(Rlltemp)-1i.*imag(Rlltemp);    % Switch elements, relation from same q
        % Dlnl(DlnlInd).dlnl(Dlnl(DlnlInd).conjInd,5)=-imag(Rlltemp)+1i.*real(Rlltemp);    % Switch elements, relation from q=-1
    end
    
    if sum(Dlnl(DlnlInd).mConjInd)
        Rlltemp=Dlnl(DlnlInd).dlnl(logical(Dlnl(DlnlInd).mConjInd),5); 
        % Dlnl(DlnlInd).dlnl(Dlnl(DlnlInd).conjInd,5)=imag(Rlltemp)-1i.*real(Rlltemp);    % Switch elements, relation from same q
        Dlnl(DlnlInd).dlnl(logical(Dlnl(DlnlInd).mConjInd),5)=-real(Rlltemp)-1i.*imag(Rlltemp);    % Switch elements, relation for m=+/-1 states
    end
    
    
    Dlnl(DlnlInd).dlnl(:,3:4)=[abs(Dlnl(DlnlInd).dlnl(:,5)) angle(Dlnl(DlnlInd).dlnl(:,5))];
%     
end