%% N2 AFPADs fitting - error analysis
%  30/11/16
%
%  (Probably) Reqs. codes in "N2_ePS_AFPADs_fitting_expt_011016.m" and "N2_ePS_XUV_AFPADs_etc_PRL_plotting_091116.m"
%  Error analysis as per "multiphoton_PADs_fitting_100811.m" (Baumert work).


%% Run init code - see "N2_ePS_AFPADs_fitting_expt_011016.m"
%  May not be necessary for some cases, depends if  AKQ, gamma,lmax,LMind,RllDlnlInd are set (or in results structure)

%% Set params for given fit result

% Set results
orb=state;

% Set numerics for chosen results
RllfitTest=RllfitTestAll(orb).RllfitTest(plotIndAll(orb));
Rllfit=RllfitTest.Rllfit;
RlltempNorm=Rllfit;



%% Find uncertainties in fitted parameters
% Loop over fit params and vary, check curvature of chi^2
% Based on old code "pad_simple_nh3_err.m"

% Rllfit=[Rllcurrent(486).Rllin; 34.71 3.56];  % In this case Rllcurrent is from "full_fit_2_180811.mat"
Rllin=Rllfit;

daj=0.1;   % Step size for test parameters

figure('name','Fitting status');

k=1;
loop=1;

for p=1:size(Rllin,1)   % Loop over parameters
    for col=1:2         % Loop over columns (mag/phase)
        Rllerr=Rllfit;
     
        m=1;
        
        for steps=-5:5  % Loop over d(aj)
        
            % Set test parameter
            Rllerr(p,col)=Rllfit(p,col)+steps.*daj;
            aj(m,k)=Rllfit(p,col);
            
            % Calculate images
            %[Ysum, BLMout]=PAD_image_fit(Rllerr, xdata, Rllindex, popAll, Ylm, YLM, YLMind, basis3D,2,params);
            %[Ysum, BLMout]=PAD_image_fit_r(Rllerr, xdata, Rllindex, popAll, Ylm, YLM, YLMind, basis3D,2,params,rFlag);
            
            % Check result
            % [BLMout, tempAll]=PAD_alignment_fit_fun_t(Rllfit, 1:length(At), Dlnl, AKQ, gamma,lmax,LMind,RllDlnlInd);
            [BLMout, tempAll]=PAD_alignment_fit_fun_t(Rllerr, 1:length(RllfitTest.tpoints), Dlnl, RllfitTest.calcRaw.AKQ, RllfitTest.calcRaw.gamma,lmax,LMind,RllDlnlInd);
                        
            % Chi^2
            % diff=imageStack_fit-Ysum;
            diff=RllfitTest.BLMin-BLMout;
            chisq(m,k)=sum(diff(:).^2);
            
            % Log results
            errorScan(m,k).BLM=BLMout;
            errorScan(m,k).Rllin=Rllerr;
            errorScan(m,k).chisq=chisq(m,k);
            
            figure(gcf);
            subplot(2,4,5:8);
            plot(loop,chisq(m,k),'x');
            hold on;
            
            m=m+1;
            loop=loop+1
          
        end
            
        save errTemp.mat loop errorScan chisq
        
        k=k+1;
    end
end

% Calculate derivatives from chisq numerically
dchisq=(chisq(1:size(chisq,1)-1,:)-chisq(2:size(chisq,1),:))./daj;
dchisq2=dchisq(1:size(dchisq,1)-1,:)-dchisq(2:size(dchisq,1),:)./daj;

sigmasq=2*(1./dchisq2);
sigma=sqrt(abs(sigmasq));

% Assign to results structure (all states)
errScanAll(orb).errorScan=errorScan;
errScanAll(orb).chisq=chisq;
errScanAll(orb).sigma=sigma;

%% Plot results

orb=1;
chisq=errScanAll(orb).chisq;

figure('color',[1 1 1]);
%plot(chisq);
surf(1:size(chisq,2),daj*[-5:5],chisq);
title(['AFPAD fitting - uncertainty analysis, ' date]);
xlabel('Parameter #');
ylabel('Deviation from fitted value');
zlabel('\chi^2');

figure('color',[1 1 1]);
%plot(chisq);
plot(daj*[-5:5],chisq);


%% Extract and reformat sigma

orb=2;

% Set to [mag phase] array
sigRow=6;
clear sigma;
sigma(:,1)=errScanAll(orb).sigma(sigRow,1:2:end);
sigma(:,2)=errScanAll(orb).sigma(sigRow,2:2:end);
sigmaRenorm=sigma;

% Renorm - ugly code from "N2_ePS_XUV_AFPADs_etc_PRL_plotting_091116.m"
if orb==1
    Rlldegen=[1 1 1 2 2 2].';
elseif orb==2   
    Rlldegen=[2 2 2 2 2 1 1].';
elseif orb==3   
    Rlldegen=[1 1 1 2 2].';    
end

% Set numerics for chosen results
RllfitTest=RllfitTestAll(orb).RllfitTest(plotIndAll(orb));
Rllfit=RllfitTest.Rllfit;
RlltempNorm=Rllfit;
RenormSum=sum((RlltempNorm(:,1).^2).*Rlldegen);

sigmaRenorm(:,1)=sqrt(sigmaRenorm(:,1).^2)./RenormSum;

% Check %age
sigmaPC=sigma./Rllfit;
sigmaPCRenorm=sigmaRenorm./Rllfit;

