%% ePS AFPADs fitting
%   11/08/16
%
%  Testing out fitting of full AFPADs (from aligned distibutions), ready for new experimental data.
%
%  AFPADs calculated using ePS_MFPADs_Blm_alignment_110816.m (or earlier versions of code)
%
%  Fitting code adapted from old butadiene work, consolidated in M:\work\photoionization_with_DV\VMI_XUV_with_Claude_040416\fitting_with_VM_N2_data_110816\old_knowledge\fitting_code
%
%  Current working dir M:\work\ePolyScat\n2_multiOrb_110416\aligned_frame_fitting_tests_110816\scripts
%
%  11/08/16 v1, adapted from butadiene_ionization_calcs_fitting_180313.m and ePS_MFPADs_Blm_alignment_110816.m

%% Set paths for Linux or Win boxes (optional!)
if isunix
    % fileBase=['/media/hockettp/StoreM/ePolyScat_stuff/no2/no2_MolPro_RLL/orb12_A1/' fileName];
    
    % Paths to some required scripts
    path(path,'/media/hockettp/StoreM/matlab');
    path(path, '/media/hockettp/StoreM/ePolyScat_stuff/scripts');
     
else
    % fileBase=['M:\work\ePolyScat\no2_testing\no2_MolPro_RLL\orb6_A1\' fileName];
    % fileBase=['M:\work\ePolyScat\no2_testing\no2_MolPro_RLL\orb12_A1\' fileName];
    % fileBase=['M:\work\ePolyScat\no2_testing\no2_MolPro_RLL\orb11_B1\' fileName];
    % fileBase=['M:\work\ePolyScat\no2_testing\no2_MolPro_RLL\orb10_A2\0.8eV_original\' fileName];
    % fileBase=['M:\work\ePolyScat\no2_testing\no2_MolPro_RLL\orb10_A2\' fileName];

    % Paths to some required scripts
    path(path,'M:\work\matlab');
    path(path, 'M:\work\ePolyScat\scripts');
    % path(path, 'M:\work\ePolyScat\ePS_scripts_2015');     % Final scripts
    path(path, 'M:\work\ePolyScat\no2_testing');            % Development scripts
    path(path, 'M:\work\ePolyScat\no2_testing\BLM_calc_testing_pt2_041215');
    
end

%% Load data to fit

%% Read in .mat file with "direct" calculations (using "ePS_MFPADs_scripts_2015")

% ePS calculations, at the moment reqd. for basic symmetry settings etc.
load('M:\work\ePolyScat\n2_multiOrb_110416\n2_3sg_0.5-30eV_A2_data_130416.mat');

%  ePS AFPAD results 11/08/16, test fitting with these
load('M:\work\ePolyScat\n2_multiOrb_110416\alignment_testing_full_case_110816\N2_AF-calcs_BLMonly_110816.mat');

%  ADMs
load('I:\work_linux_storeM_backup\alignment\alignment_codes_CZB\butadiene\butadiene_ang_mar13\butadiene_ang_incADMs_es_incSin_150313.mat');

%% Init
%  Set a bunch of parameters for AFPAD calcs, for fitting code some additional stuff is set here too.

n=1;     % Index into BLMcalc structure, if reqd.
state=1; % Used to select alignment results
Eind=12; % Used to select energy index into rlAll structure, set =1 if only a single E present.
% symGroup={1; 2:3; 4:5; 1:5}; 

symGroupInd=3;
% for symGroupInd=1:(size(rlAll,1)+1)     % Loop over symmetries to calculate for all syms, and overall sum.
    
 % *** Choose alignment calculation results to define ADMs
    calcInd=1;
    %state=3;    % =1 for ground state, =2 for excited state
    % trange=[20.1 100.1];  % Set range in ps for calc
    trange=[50 70];
    tStep=10;       % Set tStep for downsampling, in current ADMs the original steps are 0.1ps.

    slices=alignAng(calcInd).t./1e3;
    ind=find(slices>trange(1) & slices<trange(2));  
    ind=ind(1:tStep:end);
    At=slices(ind);
    
 % *** Set calculation params & generate symmetry coeffs
    lmax=4;
    %Gpath='/media/store/cs2/PAD_calcs/test/';
    %Gpath='G:\Backup\work - NRC aug 10\linux box\cs2\PAD_calcs\test\';
    % Gpath='/media/store/cs2/PAD_calcs/test/MF_PAD_conv_gamma.dat';
    % Gpath='J:\work_linux_storeM_backup\ePolyScat_stuff\circ_pol\'
    % Gpath='J:\work_linux_storeM_backup\cs2\PAD_calcs\test\MF_PAD_conv_gamma.dat';
    Gpath='M:\work\ePolyScat\no2_testing\BLM_calc_testing_pt2_041215\MF_PAD_conv_gamma.dat'
    % Gpath='M:\work\ePolyScat\no2_testing\BLM_calc_testing_pt2_041215\MF_PAD_conv_gamma_multiphoton_N1_lmax4_Kmax_20.dat'

    %*** Set matrix elements, CURRENTLY SET VIA symGroupInd
    rlThres=1e-2
    if symGroupInd<=size(rlAll,1)
        Dlnl=symm_coeffs_ePS(rlAll(symGroupInd,Eind),rlThres,0); 
        comment=['N2 ePS with alignment, ' rlAll(symGroupInd,Eind).symm ' symm only, E=' num2str(rlAll(symGroupInd,Eind).eKE) 'eV, ' date];
    else
        Dlnl=symm_coeffs_ePS(rlAll(:,Eind),rlThres,0);  
        comment=['N2 ePS with alignment, all symmetries,  E=' num2str(rlAll(1,Eind).eKE) 'eV, ' date];
    end

    % matlabpool(4);  % Open parallel computing pool, max 4 sessions in R2008b, can go higher in R2010

    %*** Init structure for results
    %n=length(BLMcalc)+1;
    % BLMcalc(n).Dlnl=Dlnl;
    % BLMcalc(n).comment=comment;

    %*** Select & filter ADMs from calc for timestep
    AKQthreshold=2e-3;
    AKQtemp=alignAng(calcInd).ADM(:,ind,state);
    ind2=find(sum(abs(AKQtemp)>AKQthreshold,2));        % Instead of thresholding per timestep, choose rows which have significant AKQ at any timestep, this should keep things smooth
    AKQ=[alignAng(calcInd).ADM_in(ind2,:) AKQtemp(ind2,:)];

   % AKQ(1,3:end)=3.545; % Scaling for A00 due to definition differences

    clear temp tempAll
    
% *** Select calculation result to fit
n=3;
% BLMin=BLMcalc(n).BLMt(:,3:end);
thres=1e-2;
% BLMin=real(BLMcalc(n).BLMt(:,3:end));   % Set values to plot
BLMin=real(BLMcalc(n).BLMt(:,ind+2));
BLMin(isnan(BLMin))=0;              % Remove NaNs, which mess up thresholding below
plotInd=find(sum(abs(BLMin),2)>thres);   % Find rows with sums > thres, plot these only
BLMin=BLMin(plotInd,:);
LMind=BLMallInd(plotInd,:);


% *** Set fitting params
Rllin=[Dlnl(1).dlnl(1:end,3:4); Dlnl(2).dlnl(1:end,3:4)];   % Set matrix elements and phases for lsqcurvefit input, Ag+Bg case
Rllin=rand(size(Rllin));    % Randomize inputs

lb=[-1*ones(size(Rllin,1),1) -pi.*ones(size(Rllin,1),1)];   % Set lb & ub
ub=[ones(size(Rllin,1),1) pi.*ones(size(Rllin,1),1)];

options=optimset('Display','iter','MaxIter',50,'TolX',1e-3,'TolFun',1e-3,'DiffMinChange',0.01);

xdata=zeros(length(At));    % Set dummy xdata

% *** Read gamma data
% gamma=pad_simple_io_MF_conv_symm_fit('/media/store/cs2/PAD_calcs/test/');
gamma=pad_simple_io_MF_conv_symm_fit(Gpath);

%% Test fitting functions

% Check result
[BLMout, tempAll]=PAD_alignment_fit_fun_t(Rllin, 1:length(At), Dlnl, AKQ, gamma,lmax,LMind);

%% Fitting - matrix elements only (fixed P(theta))

m=1;
while 1
    
    % Rllin=rand(size(Rllin));    % Randomize inputs
    Rllin=Rllfit;                 % Recycle previous results to check for convergence

    [Rllfit,resnorm,residual,exitflag,output]=lsqcurvefit(@PAD_alignment_fit_fun_t, Rllin, 1:length(At), BLMin, lb, ub, options, Dlnl, AKQ, gamma, lmax, LMind);
    
    % Check result
    [BLMout, tempAll]=PAD_alignment_fit_fun_t(Rllfit, 1:length(At), Dlnl, AKQ, gamma,lmax,LMind);
    
    % Log
    RllfitTest(m).BLMin=BLMin;
    RllfitTest(m).Rllin=Rllin;
    RllfitTest(m).lb=lb;
    RllfitTest(m).ub=ub;
    RllfitTest(m).Rllfit=Rllfit;
    RllfitTest(m).BLMout=BLMout;
    RllfitTest(m).resnorm=resnorm;
    RllfitTest(m).residual=residual;
    RllfitTest(m).exitflag=exitflag;
    RllfitTest(m).output=output;
    RllfitTest(m).calcRaw=tempAll;
    RllfitTest(m).options=options;
    
    % Save
    save('S:\fitTemp.mat','RllfitTest');
    
    % Increment
    m=m+1;
    
end
