%% ePS AFPADs fitting
%   11/08/16
%
%  Testing out fitting of full AFPADs (from aligned distibutions), ready for new experimental data.
%
%  AFPADs calculated using ePS_MFPADs_Blm_alignment_110816.m (or earlier versions of code)
%
%  Fitting code adapted from old butadiene work, consolidated in M:\work\photoionization_with_DV\VMI_XUV_with_Claude_040416\fitting_with_VM_N2_data_110816\old_knowledge\fitting_code
%
%  Current working dir M:\work\ePolyScat\n2_multiOrb_110416\aligned_frame_fitting_tests_110816\scripts
%
%  31/08/16 _expt, for using with Claude's data
%
%  11/08/16 v1, adapted from butadiene_ionization_calcs_fitting_180313.m and ePS_MFPADs_Blm_alignment_110816.m

%% Set paths for Linux or Win boxes (optional!)
if isunix
    % fileBase=['/media/hockettp/StoreM/ePolyScat_stuff/no2/no2_MolPro_RLL/orb12_A1/' fileName];
    
    % Paths to some required scripts
    path(path,'/media/hockettp/StoreM/matlab');
    path(path, '/media/hockettp/StoreM/ePolyScat_stuff/scripts');
    path(path, '/media/hockettp/StoreM/ePolyScat_stuff/no2/BLM_calc_testing_pt2_041215');
    
    path(path, '/media/hockettp/StoreM/ePolyScat_stuff/n2_2016/n2_multiOrb_110416/aligned_frame_fitting_tests_110816/scripts'); % Development fitting scripts
    
     
else
    % fileBase=['M:\work\ePolyScat\no2_testing\no2_MolPro_RLL\orb6_A1\' fileName];
    % fileBase=['M:\work\ePolyScat\no2_testing\no2_MolPro_RLL\orb12_A1\' fileName];
    % fileBase=['M:\work\ePolyScat\no2_testing\no2_MolPro_RLL\orb11_B1\' fileName];
    % fileBase=['M:\work\ePolyScat\no2_testing\no2_MolPro_RLL\orb10_A2\0.8eV_original\' fileName];
    % fileBase=['M:\work\ePolyScat\no2_testing\no2_MolPro_RLL\orb10_A2\' fileName];

    % Paths to some required scripts
    path(path,'M:\work\matlab');
    path(path, 'M:\work\ePolyScat\scripts');
    % path(path, 'M:\work\ePolyScat\ePS_scripts_2015');     % Final scripts
    path(path, 'M:\work\ePolyScat\no2_testing');            % Development scripts
    path(path, 'M:\work\ePolyScat\no2_testing\BLM_calc_testing_pt2_041215');
    
end

%% Load data to fit V1
% %  Version 1 - old ADMs (butadiene) and associated results
% 
% % ePS calculations, at the moment reqd. for basic symmetry settings etc.
% load('M:\work\ePolyScat\n2_multiOrb_110416\n2_3sg_0.5-30eV_A2_data_130416.mat');
% 
% %  ePS AFPAD results 11/08/16, test fitting with these
% load('M:\work\ePolyScat\n2_multiOrb_110416\alignment_testing_full_case_110816\N2_AF-calcs_BLMonly_110816.mat');
% 
% %  ADMs
% load('I:\work_linux_storeM_backup\alignment\alignment_codes_CZB\butadiene\butadiene_ang_mar13\butadiene_ang_incADMs_es_incSin_150313.mat');

%% Load data to fit V2
% %  Version 2 - new ADMs (N2 double pulse) and associated computational results
% 
% % ePS calculations, at the moment reqd. for basic symmetry settings etc.
% load('M:\work\ePolyScat\n2_multiOrb_110416\n2_3sg_0.5-30eV_A2_data_130416.mat');
% 
% %  ePS AFPAD results 11/08/16, test fitting with these
% load('M:\work\ePolyScat\n2_multiOrb_110416\aligned_frame_results_new_ADMs_130816\N2_3sg_alignment_new_ADMs_2_130816.mat');
% 
% %  ADMs
% % load('I:\work_linux_storeM_backup\alignment\alignment_codes_CZB\butadiene\butadiene_ang_mar13\butadiene_ang_incADMs_es_incSin_150313.mat');
% run('M:\work\ePolyScat\n2_multiOrb_110416\aligned_frame_results_new_ADMs_130816\N2_read_ADMs_130816.m');

%% Load data to fit V3 - expt.

if isunix
    % ePS calculations, at the moment reqd. for basic symmetry settings etc.
    load('/media/hockettp/StoreM/ePolyScat_stuff/n2_2016/n2_multiOrb_110416/n2_3sg_0.5-30eV_A2_data_130416.mat');

    % Data from Claude (Aug. 2016)
    load('/media/hockettp/StoreM/photoionization_with_DV/XUV_VMI_claude/CM_results_aug2016/2016_08_04_results_Claude_VMI_N2_XUV_3108016.mat');

    % Varun's ADMs (29/08/16)
    run('/media/hockettp/StoreM/ePolyScat_stuff/n2_2016/n2_multiOrb_110416/aligned_frame_results_new_ADMs_290816/N2_read_ADMs_290816.m');
    
else
    % ePS calculations, at the moment reqd. for basic symmetry settings etc.
    load('M:\work\ePolyScat\n2_multiOrb_110416\n2_3sg_0.5-30eV_A2_data_130416.mat');
    
    % Data from Claude (Aug. 2016)
    load('M:\work\photoionization_with_DV\VMI_XUV_with_Claude_040416\CM_results_aug2016\2016_08_04_results_Claude_VMI_N2_XUV_3108016.mat');

    % Varun's ADMs (29/08/16)
    run('M:\work\ePolyScat\n2_multiOrb_110416\aligned_frame_results_new_ADMs_290816\N2_read_ADMs_290816.m');
    
end
    
% Check ADMs
figure; plot(alignAng.t,alignAng.ADM.');

%% Set Blm index (if not loaded above)
% Generate index, set for all possible L,M
BLMallInd=[0 0];
for l=1:8
    for m=-l:l
        BLMallInd(end+1,:)=[l m];
    end
end



%% Init
%  Set a bunch of parameters for AFPAD calcs, for fitting code some additional stuff is set here too.

n=1;     % Index into BLMcalc structure, if reqd.
state=1; % Used to select experimental results
H=1;
Eind=15; % Used to select energy index into rlAll structure, set =1 if only a single E present.
% symGroup={1; 2:3; 4:5; 1:5}; 

symGroupInd=3;
% for symGroupInd=1:(size(rlAll,1)+1)     % Loop over symmetries to calculate for all syms, and overall sum.
    

%*** Set experimental data for fitting
% Range for fitting (ps)
trange=[4 13.5];

% Temporal axis and shift (relative to calcs.)
tShift=0.1;
taxis=input.p2(:,2)+input.p1(1,2)+tShift;  % taxis set from laser pulse info, plus additional shift to match calculations

slices=taxis;                      % Check t-index again, it may be different from AKQ t-index!
ind=find(slices>trange(1) & slices<trange(2));
% ind=ind(1:tStep:end);
tExpt=slices(ind);

BLMin=data(state,H).mean(ind,:);
BLMin=(BLMin.');
BLMin(isnan(BLMin))=0;              % Remove NaNs, which mess up thresholding below
thres=1e-2;
plotInd=find(sum(abs(BLMin),2)>thres);   % Find rows with sums > thres, plot these only - FOR EXPT DATA NOT REALLY NEEDED, BUT LEAVE IN FOR NOW.
BLMin=BLMin(plotInd,:);
LMind=[0 0; 2 0; 4 0; 6 0];              % BLMallInd(plotInd,:);    % FOR EXPT DATA SET MANUALLY AT THE MOMENT


 % *** Choose alignment calculation results to define ADMs
    calcInd=1;
    %state=3;    % =1 for ground state, =2 for excited state
    % trange=[20.1 100.1];  % Set range in ps for calc
    % trange=[50 70];
    % trange=[2.2 2.7];     % Arb. t values for Varun's calcs., might be in fs?
    % trange=[4 13.5];  % t values for Varun's updated calcs. 29/08/16, in ps.  ~[4 13.5] to skip two-pulse stuff (small time-steps) and final weird bit!
    
    
    % Select delays for calcs. by matching to expt. axis
    rFac=100;   % Set rounding, here for nearest fs
    tRes=1;
    clear Tind;
    for m=1:length(tExpt)
        % Tind(m)=find((alignAng.t<(round(tExpt(m),1)+0.1))&(alignAng.t>=(round(Ecalc(m),1)-0.1)));   % Select nearest calc. result (0.5 eV steps) via rounding and windowing - can't use round like this in R2013!
        temp=find((alignAng.t<(round(tExpt(m).*rFac)+tRes)./rFac)&(alignAng.t>=(round(tExpt(m).*rFac)-tRes)./rFac));   % Select nearest calc. result (0.5 eV steps) via rounding and windowing - can't use round like this in R2013!
        Tind(m)=temp(1);
    end    
    
%    slices=alignAng(calcInd).t;
%    ind=find(slices>trange(1) & slices<trange(2));  
%    ind=ind(1:tStep:end);
    ind=Tind;
    slices=alignAng(calcInd).t(ind);
    At=slices;
    
 % *** Set calculation params & generate symmetry coeffs
    lmax=4;
    %Gpath='/media/store/cs2/PAD_calcs/test/';
    %Gpath='G:\Backup\work - NRC aug 10\linux box\cs2\PAD_calcs\test\';
    % Gpath='/media/store/cs2/PAD_calcs/test/MF_PAD_conv_gamma.dat';
    % Gpath='J:\work_linux_storeM_backup\ePolyScat_stuff\circ_pol\'
    % Gpath='J:\work_linux_storeM_backup\cs2\PAD_calcs\test\MF_PAD_conv_gamma.dat';
    if isunix
        Gpath='/media/hockettp/StoreM/pad_calcs/PAD_codes_distro_251114/MF_PAD_conv_gamma.dat';
    else
        Gpath='M:\work\ePolyScat\no2_testing\BLM_calc_testing_pt2_041215\MF_PAD_conv_gamma.dat'
    end
    
    % Gpath='M:\work\ePolyScat\no2_testing\BLM_calc_testing_pt2_041215\MF_PAD_conv_gamma_multiphoton_N1_lmax4_Kmax_20.dat'

    %*** Set matrix elements, CURRENTLY SET VIA symGroupInd
    rlThres=1e-2
    if symGroupInd<=size(rlAll,1)
        Dlnl=symm_coeffs_ePS(rlAll(symGroupInd,Eind),rlThres,0); 
        comment=['N2 ePS with alignment, ' rlAll(symGroupInd,Eind).symm ' symm only, E=' num2str(rlAll(symGroupInd,Eind).eKE) 'eV, ' date];
    else
        Dlnl=symm_coeffs_ePS(rlAll(:,Eind),rlThres,0);  
        comment=['N2 ePS with alignment, all symmetries,  E=' num2str(rlAll(1,Eind).eKE) 'eV, ' date];
    end

    %*** Init structure for results
    %n=length(BLMcalc)+1;
    % BLMcalc(n).Dlnl=Dlnl;
    % BLMcalc(n).comment=comment;

    %*** Select & filter ADMs from calc for timestep
    AKQthreshold=2e-3;
    AKQtemp=alignAng(calcInd).ADM(:,ind,state);
    ind2=find(sum(abs(AKQtemp)>AKQthreshold,2));        % Instead of thresholding per timestep, choose rows which have significant AKQ at any timestep, this should keep things smooth
    AKQ=[alignAng(calcInd).ADM_in(ind2,:) AKQtemp(ind2,:)];

   % AKQ(1,3:end)=3.545; % Scaling for A00 due to definition differences

    clear temp tempAll
    
% *** Select calculation result to fit
% n=3;
% % BLMin=BLMcalc(n).BLMt(:,3:end);
% thres=1e-2;
% % BLMin=real(BLMcalc(n).BLMt(:,3:end));   % Set values to plot
% 
% slices=BLMcalc(n).t;                      % Check t-index again, it may be different from AKQ t-index!
% ind=find(slices>trange(1) & slices<trange(2));
% ind=ind(1:tStep:end);
% 
% BLMin=real(BLMcalc(n).BLMt(:,ind+2));
% BLMin(isnan(BLMin))=0;              % Remove NaNs, which mess up thresholding below
% plotInd=find(sum(abs(BLMin),2)>thres);   % Find rows with sums > thres, plot these only
% BLMin=BLMin(plotInd,:);
% LMind=BLMallInd(plotInd,:);


% *** Set fitting params
% Rllin=[Dlnl(1).dlnl(1:end,3:4); Dlnl(2).dlnl(1:end,3:4)];   % Set matrix elements and phases for lsqcurvefit input, Ag+Bg case
RllOriginal=[Dlnl(1).dlnl(1:end,3:4); Dlnl(2).dlnl(1:end,3:4); Dlnl(3).dlnl(1:end,3:4)];  % Specific case for N2, Su+Pu - should generalize this!

Rllin=RllOriginal(1:6,:);
Rllin=rand(size(Rllin));    % Randomize inputs

lb=[zeros(size(Rllin,1),1) -pi.*ones(size(Rllin,1),1)];   % Set lb & ub
ub=[5*ones(size(Rllin,1),1) pi.*ones(size(Rllin,1),1)];

options=optimset('Display','iter','MaxIter',50,'TolX',1e-3,'TolFun',1e-4,'DiffMinChange',1e-3);

xdata=zeros(length(At));    % Set dummy xdata

% *** Read gamma data
% gamma=pad_simple_io_MF_conv_symm_fit('/media/store/cs2/PAD_calcs/test/');
gamma=pad_simple_io_MF_conv_symm_fit(Gpath);

%% Test fitting functions

% Check result
[BLMout, tempAll]=PAD_alignment_fit_fun_t(Rllin, 1:length(At), Dlnl, AKQ, gamma,lmax,LMind);

%% Fitting - matrix elements only (fixed P(theta))

matlabpool(4);  % Open parallel computing pool, max 4 sessions in R2008b, can go higher in R2010; 
                  % For new versions, either skip this and use general parallel pool settings, or use parpool()

m=1;
while 1
    
    Rllin=rand(size(Rllin));    % Randomize inputs
    % Rllin=Rllfit;                 % Recycle previous results to check for convergence

    [Rllfit,resnorm,residual,exitflag,output]=lsqcurvefit(@PAD_alignment_fit_fun_t, Rllin, 1:length(At), BLMin, lb, ub, options, Dlnl, AKQ, gamma, lmax, LMind);
    
    % Check result
    [BLMout, tempAll]=PAD_alignment_fit_fun_t(Rllfit, 1:length(At), Dlnl, AKQ, gamma,lmax,LMind);
    
    % Log
    RllfitTest(m).BLMin=BLMin;
    RllfitTest(m).Rllin=Rllin;
    RllfitTest(m).lb=lb;
    RllfitTest(m).ub=ub;
    RllfitTest(m).Rllfit=Rllfit;
    RllfitTest(m).BLMout=BLMout;
    RllfitTest(m).resnorm=resnorm;
    RllfitTest(m).residual=residual;
    RllfitTest(m).exitflag=exitflag;
    RllfitTest(m).output=output;
    RllfitTest(m).calcRaw=tempAll;
    RllfitTest(m).options=options;
    
    % Save
%    save('S:\fitTemp.mat','RllfitTest');
    save('fitTemp.mat','RllfitTest');
    
    % Increment
    m=m+1
    
end

%% Quick test of results by Blm

% Set results to plot
plotInd=1;

figure('color',[1 1 1],'name','Rll fitting tests');
subplot(2,1,1);
plot(RllfitTest(plotInd).BLMin.');
hold on;
plot(RllfitTest(plotInd).BLMout.','--x');
% title('N2 calcs + fittings, tests 13/08/16');
title('N2 expt + fittings, tests 01/09/16');
xlabel('t/arb.');
ylabel('\beta_{LM}');

subplot(2,1,2);
plot((RllfitTest(plotInd).BLMin-RllfitTest(plotInd).BLMout).');
title('Difference, calcs vs. fit');


%% Look at fit results - Rlls summary

clear Rlltemp;
for n=1:length(RllfitTest);
    Rlltemp(:,:,n)=RllfitTest(n).Rllfit;
    resNorm(n)=RllfitTest(n).resnorm;
    exitFlag(n)=RllfitTest(n).exitflag;
end

% Relative phases
Rlltemp2=Rlltemp;
Rlltemp2(1:3,2,:)=Rlltemp2(1:3,2,:)-repmat(Rlltemp2(1,2,:),[3 1 1]);
Rlltemp2(4:6,2,:)=Rlltemp2(4:6,2,:)-repmat(Rlltemp2(4,2,:),[3 1 1]);


figure('color',[1 1 1]);

subplot(4,1,1);
plot(squeeze(Rlltemp(:,1,:))');
title('Magnitudes');
xlabel('Fit #');
ylabel('|R|');

subplot(4,1,2);
plot(squeeze(Rlltemp(:,2,:))');
title('Phases');
xlabel('Fit #');
ylabel('arg(R)/rad');

subplot(4,1,3);
plot(squeeze(Rlltemp2(:,2,:))');
title('Phases');
xlabel('Fit #');
ylabel('arg(R)/rad');



subplot(4,1,4);
plot(resNorm);
hold on;
plot(exitFlag,'-x');

%% Utils

% Renorm relative phases

% RlltempNorm=RllfitTest(1).Rllfit;
RlltempNorm=RllOriginal;

RlltempNorm(1:3,2)=RlltempNorm(1:3,2)-repmat(RlltempNorm(1,2),[3 1]);
RlltempNorm(4:6,2)=RlltempNorm(4:6,2)-repmat(RlltempNorm(4,2),[3 1]);
