function [B00out, calcOut]=PAD_alignment_fit_fun_t(Rllin, At, Dlnl, AKQ, gamma, lmax)
%
% Function for calculating PADs for fitting to alignment data
% Operation as per "butadiene_ionization_calcs_070313.m", except for use with lsqcurvefit, see "butadiene_ionization_calcs_fitting_180313.m" for calling code
%
% INPUTs:
%   - Rllin, matrix elements to be determined, note these are converted into Dlnl symmetrized elements
%   - Time index vector for AKQ(t)
%   - Symmetrization coeffs & matrix elements
%   - AKQ alignment moments
%   - gamma data
%   - lmax
%
% OUTPUT:
%   - B00 only, for fit
%   - tempAll, full result for later checking/testing
%
%
% 20/03/13  _t, moved parfor to pad_simple_MF_conv_symm_fit_t to speed up looping over AKQ(t)
% 18/03/13  version 1

% Reformat input Rllin, this is currently set for Ag+Bg
ind=size(Dlnl(1).dlnl,1);
Dlnl(1).dlnl(:,3:4)=Rllin(1:ind,:);
Dlnl(2).dlnl(:,3:4)=Rllin((ind+1):end,:);

% Calculate BLMs (at the moment set for all BLM, but could do B00 only for speed)
%parfor m=1:length(At)
    
    %*** Calculate ionization
    calcOut=pad_simple_MF_conv_symm_fit_t(Dlnl,AKQ,gamma, lmax);  % At the moment get structure returned here, may want to modify either function output or later processing to get more usable structure
                                                            % c.f. code for Baumert stuff for better storing & sorting methods
%    tempAll(m).BLM=temp.BLM;
%    tempAll(m).BLMnorm=temp.BLMnorm;
    
%end

% Extract B00 from calc results
%for m=1:length(At)
%    B00out(m)=tempAll(m).BLM(1,3);
%end
    
% Extract B00 from calc results
for m=1:length(At)
   B00out(m)=calcOut.BLMt(m).BLM(1,3);
end