function [BLMout, calcOut]=PAD_alignment_fit_fun_t(Rllin, At, Dlnl, AKQ, gamma, lmax, LMind)
%
% Function for calculating PADs for fitting to alignment data
% Operation as per "butadiene_ionization_calcs_070313.m", except for use with lsqcurvefit, see "butadiene_ionization_calcs_fitting_180313.m" for calling code
%
% INPUTs:
%   - Rllin, matrix elements to be determined, note these are converted into Dlnl symmetrized elements
%   - Time index vector for AKQ(t)
%   - Symmetrization coeffs & matrix elements
%   - AKQ alignment moments
%   - gamma data
%   - lmax
%   - LMind, LM index of input, to use for output formatting & checking
%
% OUTPUT:
%   - BLM for fit
%   - tempAll, full result for later checking/testing
%
%
% 11/08/16  General version for all Blm
% 20/03/13  _t, moved parfor to pad_simple_MF_conv_symm_fit_t to speed up looping over AKQ(t)
% 18/03/13  version 1

% Reformat input Rllin, this is currently set for Ag+Bg
ind=size(Dlnl(1).dlnl,1);
Dlnl(1).dlnl(:,3:4)=Rllin(1:ind,:);
Dlnl(2).dlnl(:,3:4)=Rllin((ind+1):end,:);

% Calculate BLMs (at the moment set for all BLM, but could do B00 only for speed)
% parfor m=1:length(At)
    
    %*** Calculate ionization
    calcOut=pad_simple_MF_conv_symm_fit_t(Dlnl,AKQ,gamma, lmax);  % At the moment get structure returned here, may want to modify either function output or later processing to get more usable structure
                                                            % c.f. code for Baumert stuff for better storing & sorting methods
%    tempAll(m).BLM=temp.BLM;
%    tempAll(m).BLMnorm=temp.BLMnorm;
%    
% end

% Extract B00 from calc results
%for m=1:length(At)
%    B00out(m)=tempAll(m).BLM(1,3);
%end
    
% Extract B00 from calc results
% for m=1:length(At)
%    B00out(m)=calcOut.BLMt(m).BLM(1,3);
% end

% Extract BLM from calc results, no LM reindexing
for m=1:length(At)
   BLMout(:,m)=calcOut.BLMt(m).BLM(:,3);
end
BLMout=real(BLMout);


% % *** Extract BLM from calc results to match input LMind for fitting - THIS IS CURRENTLY THE WRONG CODE, too complex
% % Extract BLMs for each timestep & re-index (allows for variable BLMs over calculation steps)
% % Code from "multiphoton_PADs_fitting_analysis_211212.m"
% 
% % Generate index, set for all possible L,M
% BLMallInd=LMind;
% 
% % Loop over results and assign BLM for each pol. state to correct index - using wrong code here, which was for case of multiple pages, each for (t).  Bodged it...
% BLMout=zeros(length(LMind),length(BLMcalc(:)));
% for n=1:length(BLMcalc(:))
%     
%     BLMcalc(n).BLMt=NaN.*zeros(size(BLMallInd,1),size(BLMcalc(n).calcRaw,2));  % Init as NaNs
%     BLMcalc(n).BLMtNorm=BLMcalc(n).BLMt;
%     
%     BLMcalc(n).BLMt(:,1:2)=BLMallInd;
%     BLMcalc(n).BLMtNorm(:,1:2)=BLMallInd;
%     
%     for m=1:length(BLMcalc(n).calcRaw)
%         for p=1:size(BLMcalc(n).calcRaw(m).BLM,1)
%             ind=find(BLMcalc(n).calcRaw(m).BLM(p,1)==BLMallInd(:,1) & BLMcalc(n).calcRaw(m).BLM(p,2)==BLMallInd(:,2));  % Faster way of indexing??
%             BLMcalc(n).BLMt(ind,m+2)=BLMcalc(n).calcRaw(m).BLM(p,3);
%             BLMcalc(n).BLMtNorm(ind,m+2)=BLMcalc(n).calcRaw(m).BLMnorm(p,3);
%         end
%     end
% 
%     BLMout(:,n)=BLMcalc(n).BLMt(:,3);
%     
%     % Plot
% %     figure('name',['Calculation results, n=' num2str(n)]);
% %     plotInd=find(sum(BLMcalc(n).BLMtNorm(:,3:end),2));   % Find rows with non-zero sums, plot these only
% %     %plotInd2=find(sum(BLMcalc(n).AKQin>BLMcalc(n).AKQthreshold,2));   % Find rows with any AKQ(t)>threshold
% %     subplot(3,1,1);
% %     plot(BLMcalc(n).t,BLMcalc(n).BLMt(plotInd,3:end));
% %     subplot(3,1,2);
% %     plot(BLMcalc(n).t,BLMcalc(n).BLMtNorm(plotInd,3:end));
% %     subplot(3,1,3);
% %     %plot(BLMcalc(n).t,BLMcalc(n).AKQinThres(:,3:end),'--');
% %     plot(BLMcalc(n).t,AKQ(:,3:end),'--');
% 
% end
% calcOut=BLMcalc;
