%% Butadiene alignment trace ionization dynamics fitting
%  Code based on "butadiene_alignment_calcs_070313.m", adapted for fitting
%  18/03/13
%
%  TRY:
%     - Fitting dynamics from know (generated) trace, single P(theta) (beam conditions fixed)
%     - Fitting dynamics from know (generated) trace, including P(theta) (beam conditions fitted)
%     - Fitting experimental data!

%% Load calc results

load /media/store/alignment/alignment_codes_CZB/butadiene/butadiene_ang_mar13/butadiene_ang_incADMs_es_Trot_250313.mat

%% Init params

% matlabpool 4

% *** Choose alignment calculation results to define ADMs
calcInd=20;
state=3;    % =1 for ground state, =2 for excited state cos^2 excitation, =3 for excited state sin^2 excitation
%trange=[50 70];  % Set range in ps for calc
trange=[57 64];  % Set range in ps for calc

slices=alignAng(calcInd).t./1e3;
ind=find(slices>=trange(1) & slices<=trange(2));  
At=slices(ind);

%*** Select & filter ADMs from calc for timestep
AKQthreshold=2e-3;
AKQtemp=alignAng(calcInd).ADM(:,ind,state);
ind2=find(sum(abs(AKQtemp)>AKQthreshold,2));        % Instead of thresholding per timestep, choose rows which have significant AKQ at any timestep, this should keep things smooth
AKQ=[alignAng(calcInd).ADM_in(ind2,:) AKQtemp(ind2,:)];

AKQ(1,3:end)=3.545; % Scaling for A00 due to definition differences

% *** Set calculation params & generate symmetry coeffs
lmax=4;
Dlnl=symm_coeffs_c2h(lmax,0);

Dlnl(1).q=0;        % Case for butadiene s2->d0
Dlnl(2).q=[1 -1];
Dlnl(4)=[];         % Remove unused symm. states.
Dlnl(3)=[];

% *** Select B00 calculation result
n=1;
B00in=BLMcalc(n).BLMt(1,3:end);


% *** Set fitting params
Rllin=[Dlnl(1).dlnl(1:end,3:4); Dlnl(2).dlnl(1:end,3:4)];   % Set matrix elements and phases for lsqcurvefit input, Ag+Bg case
Rllin=rand(size(Rllin));    % Randomize inputs

lb=[zeros(size(Rllin,1),1) -0.01.*ones(size(Rllin,1),1)];   % Set lb & ub
ub=[ones(size(Rllin,1),1) 0.01.*ones(size(Rllin,1),1)];

options=optimset('Display','iter','MaxIter',50,'TolX',1e-4,'TolFun',1e-4,'DiffMinChange',0.01);

xdata=zeros(length(At));    % Set dummy xdata

% *** Read gamma data
gamma=pad_simple_io_MF_conv_symm_fit('/media/store/cs2/PAD_calcs/test/');

%% Fitting - matrix elements only (fixed P(theta))

% m=1;
while 1
    
    Rllin=rand(size(Rllin));    % Randomize inputs

    [Rllfit,resnorm,residual,exitflag,output]=lsqcurvefit(@PAD_alignment_fit_fun, Rllin, 1:length(At), B00in, lb, ub, options, Dlnl, AKQ, gamma, lmax);
    
    % Check result
    [B00out, tempAll]=PAD_alignment_fit_fun(Rllfit, 1:length(At), Dlnl, AKQ, gamma,lmax);
    
    % Log
    RllfitTest(m).B00in=B00in;
    RllfitTest(m).Rllin=Rllin;
    RllfitTest(m).lb=lb;
    RllfitTest(m).ub=ub;
    RllfitTest(m).Rllfit=Rllfit;
    RllfitTest(m).B00out=B00out;
    RllfitTest(m).resnorm=resnorm;
    RllfitTest(m).residual=residual;
    RllfitTest(m).exitflag=exitflag;
    RllfitTest(m).output=output;
    RllfitTest(m).calcRaw=tempAll;
    RllfitTest(m).options=options;
    
    % Save
    save fitTemp.mat RllfitTest;
    
    % Increment
    m=m+1;
    
end
    
     

%% Check result
tic
B00out2=PAD_alignment_fit_fun(Rllfit, 1:length(At), Dlnl, AKQ, gamma,lmax);
toc
%%

figure;
plot(At,B00in);
hold on;
plot(At,B00out,'xr');


%% Plot single fit result

n=15;
figure;
plot(RllfitTest(n).B00in);
hold on;
plot(RllfitTest(n).B00out,'xr');


%% Process batch of fits

for n=1:length(RllfitTest)
    RllfitAll(:,:,n)=RllfitTest(n).Rllfit;
    B00all(:,n)=RllfitTest(n).B00out;
    B20all(:,n)=RllfitTest(n).BLMtNorm(7,3:end);    % First use code in "butadiene_ionization_calcs_070313.m" to consolidate BLMs from calcRaw.
    resnorm(n)=RllfitTest(n).resnorm;
end

%% Plot fit results

figure('color',[1 1 1],'name','fit batch summary');
subplot(3,1,1);
plot([squeeze(RllfitAll(1,1,:)) squeeze(RllfitAll(2,1,:)) squeeze(RllfitAll(3,1,:)) squeeze(sum(RllfitAll(1:3,1,:).^2))]);
hold on;
plot(repmat([0.7 0.7 0.7 3*(0.7^2)].',1,size(RllfitAll,3)).','--');
title('Ag, Fitted Dln and \Sigma(Dln^2), dashed lines show input values');
xlabel('fit #');
ylabel('Dln');
legend('l=0, m=0', 'l=2, m=0', 'l=2, m=+/-2', 'sum(dln^2)');
grid on;

subplot(3,1,2);
plot(squeeze(RllfitAll(4,1,:)),'r');
hold on;
plot(repmat((0.5),1,size(RllfitAll,3)),'--r');
title('Bg, Fitted Dnl');
xlabel('fit #');
grid on;

subplot(3,1,3);
plot(resnorm);
title('Resnorm');
xlabel('fit #');
grid on;    
    
    
figure('color', [1 1 1],'name','fit batch summary, B20'); 
surf(At,1:63,abs(B20all).'); 
shading interp;
xlabel('t/ps');
ylabel('fit #');
zlabel('\beta_{20}');
title('Alignment fit testing 18/03/13, \beta_{20}');

%% Test new code with adapted looping

% matlabpool(4);
[B00out, calcOut]=PAD_alignment_fit_fun_t(RllfitTest(1).Rllfit, 1:length(At), Dlnl, AKQ, gamma, lmax);

%% Test code with P(theta) selection (NOTE - this requires tInd, state and alignAng data to be available in base workspace)

AKQparams=[4 0.4 1];
[B00out, calcOut]=PAD_alignment_fit_fun_t_TI(RllfitTest(1).Rllfit, 1:length(At), Dlnl, AKQparams, gamma, lmax);

%% Test code with P(theta) selection, ratio version (NOTE - this requires tInd, state and alignAng data to be available in base workspace)

AKQparams=[3 0.4 1];
[B00out, calcOut]=PAD_alignment_fit_fun_t_TI_ratio([0.7 0.5], 1:length(At), Dlnl, AKQparams, gamma, lmax);


%% Fitting - RATIO of Ag to Bg matrix elements only, inc. P(I,tau,T)

% Load & set experimental data to fit (see first cell for setting calculation data)
% load /media/store/butadiene/mag_bott/MB_alignment_061212/butadiene_alignment_131212.mat % OR use code in mb_data_processing_alignment_dec12.m to load specific datasets
% dataIn=[dataAll.delays sum(dataAll.scantrace).'];   % set data to fit after read-in of single dataset
% dataIn=flipud(dataIn);                              % transform to correct time axis
% dataIn(:,1)=-dataIn(:,1);
% 
% B00in=dataIn(:,2)./max(dataIn(:,2));          % In this case expt & calc have same steps (100fs), and range already set.  More generally might need to use interp1 & set range here
% B00in=B00in.';
%
% DATA only file: load /media/store/alignment/alignment_codes_CZB/butadiene/butadiene_ang_mar13/butadiene_data_D12120412_fitting_260313.mat
%
% *** PROCESSED VERSION: load /media/store/butadiene/ionization_calcs/butadiene_alignment_model_fitting_init_Trot_D12120412_250313.mat;
% 

normFlag=logical(1);    % Set =1 to renormalize mat. elements to sum(square)=1 (currently implemented in 2-photon version only)

% Set time range, assumes all calcs have same t axis & same axis as expt.
slices=alignAng(1).t./1e3;
ind=find(slices>=trange(1) & slices<=trange(2));  
indExp=find(dataIn(:,1)>=trange(1) & dataIn(:,1)<=trange(2));
At=slices(ind);
xdata=zeros(length(At));    % Set dummy xdata

% Parameter generation, set for ranges to fit
clear AKQparams
n=1;
for I=5
    for tau=400    %300:100:600 %:100:700            %0.2:0.1:0.6
        for Trot=2  %:2:4
            AKQparams(n,:)=[I tau Trot];
            n=n+1;
        end
    end
end

lb=[0 0 0 0 0 0 0 -1.5];   % For ratios case only have 2 variables, scaling factor, baseline offset
ub=[1 1 10 10 1 10 10 1.5];

%for m=1:(n-1)
while 1    
    Rllin=rand(8,1);    % Randomize inputs
    %Rllin(3)=Rllin(3)*1e3;  % Scaling factor

    [Rllfit,resnorm,residual,exitflag,output]=lsqcurvefit(@PAD_alignment_fit_fun_t_TI_ratio_2photon_toffset, Rllin, 1:length(At), B00in(indExp), lb, ub, options, Dlnl, AKQparams(1,:), gamma, lmax,normFlag);
    
    % Check result
    [B00out, tempAll]=PAD_alignment_fit_fun_t_TI_ratio(Rllfit, 1:length(At), Dlnl, AKQparams(1,:), gamma, lmax);
    
    % Log
    RllfitTest(m).B00in=B00in;
    RllfitTest(m).Rllin=Rllin;
    RllfitTest(m).lb=lb;
    RllfitTest(m).ub=ub;
    RllfitTest(m).Rllfit=Rllfit;
    RllfitTest(m).B00out=B00out;
    RllfitTest(m).resnorm=resnorm;
    RllfitTest(m).residual=residual;
    RllfitTest(m).exitflag=exitflag;
    RllfitTest(m).output=output;
    RllfitTest(m).calcRaw=tempAll;
    RllfitTest(m).options=options;
    RllfitTest(m).AKQparams=AKQparams(1,:);
    
    % Save
    save fitTemp.mat RllfitTest;
    
    % Increment
    m=m+1
    % m
end

%% Process batch of fits - ratios

clear RllfitAll B00all B20all resnorm
for n=1:length(RllfitTest)
    RllfitAll(:,n)=RllfitTest(n).Rllfit;
    B00all(:,n)=RllfitTest(n).B00out;
    % B20all(:,n)=RllfitTest(n).BLMtNorm(7,3:end);    % First use code in "butadiene_ionization_calcs_070313.m" to consolidate BLMs from calcRaw.
    resnorm(n)=RllfitTest(n).resnorm;
    lText{n}=['I= ' num2str(RllfitTest(n).AKQparams(1)) ', \tau=' num2str(RllfitTest(n).AKQparams(2))];
end

figure('color',[1 1 1],'name','Alignment data fitting ratios');
subplot(3,1,1);
plot(RllfitAll.');
title('Fitted parameters');
legend('|Ag|','|Bg|','cos^2 ex','sin^2 ex','offset','scale factor');
xlabel('Parameter set');
ylabel('|D|/arb.');
grid on;

subplot(3,1,2);
plot(resnorm);
title('Residual');
xlabel('Parameter set');
grid on;

subplot(3,1,3);
plot(At-RllfitAll(7),B00all./repmat(sum(B00all),size(B00all,1),1),'--');
hold on;
plot(At,B00in(indExp)./sum(B00in(indExp)),'g');
legend(lText, 'Input');
grid on;
title('All fits');
xlabel('t/ps');
ylabel('B00');

figure('color', [1 1 1],'name','fit batch summary, B00'); 
surf(At,1:size(B00all,2),(B00all).'); 
shading interp;
hold on;
plot3(At,size(B00all,2)*ones(length(At),1),B00in(indExp));
xlabel('t/ps');
ylabel('Parameter set');
zlabel('\beta_{00}');
title('Alignment fit testing 21/03/13, D12120412.TRP dataset, \beta_{00}');

%% Plot single
n=2;
figure; 
plot(At-RllfitTest(n).Rllfit(8),RllfitTest(n).B00out); 
hold on; 
plot(At,B00in(indExp).*2,'r');