function band=pad_simple_MF_conv_symm_fit_t(Dlnl,AKQ,gamma,lmax)

% Function to calculate MF-PADs using symmetrized dipole matrix elements and phases, convolved with LF axis distribution defined by A(K,Q).
% Based on various old codes codes, primarily "pad_simple_nh3_b_NS" and accompanying functions.
%
% By Paul Hockett
% paul.hockett@nrc.ca
%
% 20/03/13  _symm_fit_t, for speed changed to internal parfor loop over AKQ(t), saves repeating sorting of gamma params.  THIS CHANGE may have broken some other stuff!
% 18/03/13  _symm_fit, as _symm version but adapted slightly for fitting: gamma data passed to funtion for speed, similarly lmax passed to avoid too much overhead
% 15/03/13  _symm, branched from v5, adapted to use symmetrized dipole matrix elements as per Underwood & Reid JCP 113 1067 2000.
%                 NOTE - for doubly degnerate states mu index is currently not used, instead just have 2*h terms & sum over mu is implicit in sum over h
% 19/10/10  v5, this version has signed lambda and q terms
% 13/10/10  v4, changed indexing of rlnl to allow for proper treatment of q,q' terms.  Formatting of rlnl now similar to ePolyScat matrix elements [l m q rl nl]
% 15/03/10  v3, added in terms in q, q' which were previously summed over.  This should correct problem with alignment calculation!  Note that (q,q') treatment still not generalized fully.
% 14/10/09  v2, explicitly added in term in Q to try and iron out bugs
% 09/10/09  v1, adapted from "pad_simple_MF_conv_t.m", dated 30/09/09 and "pad_simple_MF.m"
%
% INPUTS:
%   Dlnl - symmetrize radial dipole matrix elements & phases. Format Dlnl(sym).dlnl=[h l Dl nl; ...] where sym indexes irreps of the point group, and implicitly defines allowed q.
%           Dlnl also has elements Dlnl(sym).sym= irrep, e.g. 'Ag', Dlnl(sym).q= allowed q for irrep, will be =0 or [1 -1] 
%           and bhllam, symmetrization coefficients for the Dl.  Format bhllam{h}[l lam bhlam;...].
%   AKQ - alignment moments, form is [K, Q, AKQ(t);...].  These are assumed to be the same for all intermediate states. Note also that Q=+/-M
%   gamma - look-up table of gamma parameters
%   lmax - max partial wave ang. mom., max L = 2*lmax
%
% OUTPUTS:
%   band(i) - structure containing all data including gamma values read from file, all betas calculated for states(i)
%             and summed values (unnormalized)



%% Initilization

%assign radial dipole matrix elements and phases (NB now passed in function call)
%indexing to match pad calculating code later, l+abs(lambda)+i where i defines index offset and is incremented according
%to l-wave component

%gamma=pad_simple_io(path); %call IO function to read in gamma calculations for required states - IN THIS VERSION pass gamma

%Define various angular momentum parameters for calculation
L=0:1:(2*lmax);    
M=-lmax:lmax;

% MAY NEED TO SET lmax, line 249

plot3D=0;
plot2D=0;

%% Calculate PADs and plot

%band=betaBandCalc(rlnl,band, L, M)    %call function to calculate beta values for each band(h)
%[band.BLM, band.rl1,band.rl2]=betaCalc(rlnl,band.gamma,L,M,q); 
%[band.BLM, band.rl1, band.rl2, band.a]=betaCalc(Dlnl,AKQ,gamma,L,M); 
band.BLMt=betaCalc(Dlnl,AKQ,gamma,L,M); %NB same as above, but without storing rl1 and rl2 variables
band.gamma=gamma;
%band.BLMnorm=[real(band.BLM(:,1:2)) band.BLM(:,3)./band.BLM(1,3)];
band.Dlnl=Dlnl;
band.AKQ=AKQ;
%if exists(comment)     % Need to change function call to use varargin to get this working correctly
%    band.comment=comment;
%else
%    band.comment='';
%end

if plot3D
    BLM=[real(band.BLM(:,1:2)) band.BLM(:,3)];
    [I, theta,phi]=pad_sph([BLM zeros(size(BLM,1),1)],50);
end

% 
% if plot2D
%     % Plot final PAD(s) in 2D polar coords
%     l_plot_v4([ones(size(BLMabs,1),1) BLMabs(:,2:4) label(:,3:4)], 1:size(BLMabs,1), 3, 0, 0);
% 
%     set(gcf, 'PaperType', 'A4','PaperOrientation', 'landscape','PaperPositionMode', 'manual','PaperUnits', 'centimeters','PaperPosition', [0.63 0.63 28.4 19.7]);
%     annotation(gcf,'textbox','String',{['\bfX(0) \rightarrow B(4) \rightarrow X(4, N_K )'],['\rm' date],[],['r_{s\sigma}=' num2str(rlnl(1,1)) ', \eta_{s\sigma}=' num2str(rlnl(1,2))],['r_{p\sigma}=' num2str(rlnl(2,1)) ', \eta_{p\sigma}=' num2str(rlnl(2,2))],['r_{p\pi}=' num2str(rlnl(3,1)) ', \eta_{p\pi}=' num2str(rlnl(3,2))], ['r_{d\sigma}=' num2str(rlnl(4,1)) ', \eta_{d\sigma}=' num2str(rlnl(4,2))], ['r_{d\pi}=' num2str(rlnl(5,1)) ', \eta_{d\pi}=' num2str(rlnl(5,2))], ['r_{d\delta}=' num2str(rlnl(6,1)) ', \eta_{d\delta}=' num2str(rlnl(6,2))], ['r_{f\sigma}=' num2str(rlnl(7,1)) ', \eta_{f\sigma}=' num2str(rlnl(7,2))], ['r_{f\pi}=' num2str(rlnl(8,1)) ', \eta_{f\pi}=' num2str(rlnl(8,2))], ['r_{f\delta}=' num2str(rlnl(9,1)) ', \eta_{f\delta}=' num2str(rlnl(9,2))],['r_{f\phi}=' num2str(rlnl(10,1)) ', \eta_{f\phi}=' num2str(rlnl(10,2))]},'FitHeightToText','off','Position',[0.4139 0.5252 0.151 0.197],'LineStyle','none');
% 
%     figure('color',[1 1 1]);
%     bar(BLMabs(:,1));
% end
end

%% IO function
function gamma=pad_simple_io(path)

% IO function for "pad_simple.m"
% Based on "betaSimFit_3_io.m" developed for acetylene calculations, dated 14/05/07.
% Adapted to take {states} as input, each row defines |ji ki>|j+ k+>{excitation branch, 0=q, 1=r, 2=p}

%read in gamma values - very inelegant but i'm crap at IO
    
% fName=['MF_PAD_conv_gamma'];
% [path 'MF_PAD_conv_gamma.dat']
% %clear c1 c2 c3 c4 c5 c6 c7 c8 c9 c10 c11 c12;
% [c1,c2,c3,c4,c5,c6,c7,c8,c9,c10, c11, c12]=textread([path fName '.dat'], '%f %f %f %f %f %f %f %f %f %f %f %f', 'headerlines',8);
% gamma(1:length(c1),1:12)=[c1 c2 c3 c4 c5 c6 c7 c8 c9 c10 c11 c12]; %indexing as n+1 to allow for n=0 cases

end


%% Calculating functions


% BLMabs=BLMt(:,1);   %store unnormalized betas
% %renormalize BLMt
% BLMt=BLMt./repmat(BLMt(:,1),1,size(BLMt,2));
% BLMabs(:,2:size(BLMt,2))=BLMt(:,2:end);



function [betaOut,rl1,rl2,a]=betaCalc(Dlnl,AKQ,gamma,L,M)

% Code to calculate beta values from input dipole matrix elements and phases, and gamma values.
% Adapted from betaCalc_3_lam.m, dated 18/05/07

% 15/03/13 - adapted for symmetrized version of code
% [... later adapted for AKQ version...]
% 13/03/08 - version 1

%Inputs:
% gamma is table of gamma values, along with L,M,l1,lambda1... as output by
% C code
% rl and nl are radial dipole matrix elements and phases for each (l,
% lambda) pair, one row per l and columns for lambda.

%Output is [B00 B20 B40] for given ion rot state
%betaOut=zeros(1,length(L));    %case for no gamma values


if(~isempty(gamma))   %check gamma exists
    %set up vectors for calculation from inputs
    rl1=zeros(length(gamma),1);
    rl2=rl1;
    nl1=rl1;
    nl2=nl1;
%    rlRow=0;    %variable to synch loop with correct rl term
    
%     for l=0:1:4        %loop over l waves      %should move this loop to calling function for efficiency
%        for lambda=0:l  %loop over lambda
%           for q=0:1    %loop over q=0, +/-1
%            
%                 rlRow=l+abs(lambda)+n;
%                 rlPage=q+1;
%                 
%                 rl1=(gamma(:,4)==l).*(abs(gamma(:,5))==lambda)*rlnl(rlRow,1,rlPage)+rl1;
%                 rl2=(gamma(:,6)==l).*(abs(gamma(:,7))==lambda)*rlnl(rlRow,1,rlPage)+rl2;
% 
%                 nl1=(gamma(:,4)==l).*(abs(gamma(:,5))==lambda)*rlnl(rlRow,2,rlPage)+nl1;
%                 nl2=(gamma(:,6)==l).*(abs(gamma(:,7))==lambda)*rlnl(rlRow,2,rlPage)+nl2;
% 
%                 %test(n+lambda,:)=[l lambda rlRow rlnl(rlRow,1)]
%            end
%         end
%         n=n+l;
%     end
    %tic    
    for dlSym=1:length(Dlnl)    % Loop over input symmetries NOTE - for doubly degnerate states mu index is currently not used, instead just have 2*h terms & sum over mu is implicit in sum over h
        
        if abs(sum(Dlnl(dlSym).dlnl(:,3)))>1e-5  % Check sym. state has amplitudes set ~=0
        
            for rlRow=1:size(Dlnl(dlSym).dlnl,1)  % Loop over input matrix elements
                h=Dlnl(dlSym).dlnl(rlRow,1);           % Set QNs as input
                l=Dlnl(dlSym).dlnl(rlRow,2);

                for qInd=1:length(Dlnl(dlSym).q)    % Loop over allowed q for given symmetry
                    q=Dlnl(dlSym).q(qInd);

                    for lamRow=1:size(Dlnl(dlSym).bhlm{h},1)  % Loop over bhllam terms (single h & l can have multiple lambda components)
                        lambda=Dlnl(dlSym).bhlm{h}(lamRow,2);

                        rl1=(gamma(:,4)==l).*(gamma(:,5)==lambda).*(gamma(:,11)==q)*conj(Dlnl(dlSym).dlnl(rlRow,3))*conj(Dlnl(dlSym).bhlm{h}(lamRow,3))+rl1;
                        rl2=(gamma(:,6)==l).*(gamma(:,7)==lambda).*(gamma(:,12)==q)*Dlnl(dlSym).dlnl(rlRow,3)*Dlnl(dlSym).bhlm{h}(lamRow,3)+rl2;

                        nl1=(gamma(:,4)==l).*(gamma(:,5)==lambda).*(gamma(:,11)==q)*Dlnl(dlSym).dlnl(rlRow,4)+nl1;
                        nl2=(gamma(:,6)==l).*(gamma(:,7)==lambda).*(gamma(:,12)==q)*Dlnl(dlSym).dlnl(rlRow,4)+nl2;

                    end
                end
            end
        end
    end
    %toc

    %tic
    parfor tInd=3:size(AKQ,2)  % Loop over input AKQ terms
        
        %betaOut(tInd-2).BLM=[0 0 0];
        BLMtemp=[0 0 0];
        
        % Set alignment terms
        a=zeros(length(gamma),1);
        for term=1:size(AKQ,1)
            a=(gamma(:,9)==AKQ(term,1)).*(gamma(:,10)==-AKQ(term,2))*AKQ(term,tInd)+a;  % Set alignment terms for given K,Q(=M,-M).  Note sum over P implicitly included here
        end

        phase=(-1i).^(gamma(:,6)-gamma(:,4));  %NOTE phase term in Underwood & Reid 2000 seems incorrect and gives -ve B00 values. This is term from Reid & Zare 1991.
        %phase=(-1i).^(gamma(:,4)-gamma(:,6));

        %Calculate BLM terms.  Note that summation over (q,q') is implicitly included in sum over M, also terms are selected by |q| in rl1 etc in loop above.
        n=1;
        for l=L;
            for m=-l:l;
                betaSubset=a.*phase.*(gamma(:,2)==l).*(gamma(:,3)==m).*gamma(:,1).*rl1.*rl2.*exp(1i*(nl1-nl2)); %.*(abs(gamma(:,11))==0).*(abs(gamma(:,12))==0);
                BLM=(sum(betaSubset));
                if abs(BLM)>1e-10
                    %betaOut(tInd-2).BLM(n,:)=[l m BLM];
                    BLMtemp(n,:)=[l m BLM];
                    n=n+1;
                end
                %[l m BLM(l+1+m)]
            end
        end
        
        betaOut(tInd-2).BLM=BLMtemp;
        betaOut(tInd-2).BLMnorm=[real(BLMtemp(:,1:2)) BLMtemp(:,3)./BLMtemp(1,3)];  % Normalized BLM
        
    end
    %betaOut=BLM(L+1)
    %toc
end

end
