function Xhulm=symm_coeffs_ePS(rlAll, rlThres, Xflag)
%
% Function to set bhlm symmetry coeffs for Blm calculations using ePolyScat results (for compatibility with existing symmetrized code)
% In this case the coeffs are already incorporated in the mat. elements, so this just lists allowed symmetries & (l,m,q) components
% See "symm_coeffs_c2h.m" for example where symmetrization is required.
% 
% Input: 
%           rlAll       ePS results strucuture at a single energy, with one element per symmetry
%           rlThres     Threshold for mat. elements
%           Xflag=0, 1  set to 1 to calculate X(theta,phi) harmonic explicitly
%           
% Output: Xhulm, structure with calculation results
%
% 08/10/16  v2, added some extra functionality to output for checking symmetrizations

%*** Grab symmetries & partial waves from rlAll (1 page per sym. and q)
n=1;
for symInd=1:size(rlAll,1)      % Loop over symmetries 
    % Xhulm(n).p=rlAll(symInd).p;
       
    if ~isempty(rlAll(symInd).rlnl1)
        q=unique(rlAll(symInd).rlnl1(:,5));
        
        for qInd=1:length(q)    % Loop over allowed q terms.  For compatibility with existing code, split symmetry by q components
            rlInd=(rlAll(symInd).rlnl1(:,5)==q(qInd))&(rlAll(symInd).rlnl1(:,3)>rlThres);
            Xhulm(n).rlInd=rlInd;
            Xhulm(n).dlnl(:,2:4)=[rlAll(symInd).rlnl1(rlInd,1) rlAll(symInd).rlnl1(rlInd,3) rlAll(symInd).rlnl1(rlInd,4)];   % Set mat. elements in mag, phase form.
            Xhulm(n).dlnl(:,5)=[rlAll(symInd).rawIdy1(rlInd,5)];   % Set mat. elements in complex form.
            Xhulm(n).dlnl(:,1)=1:length(Xhulm(n).dlnl(:,2));
            
            Xhulm(n).rlAllIn=[rlAll(symInd).rlnl1(rlInd,:) rlAll(symInd).rawIdy1(rlInd,5)];   % Index full input for easy reference later!

            % Set bhlm to one entry per partial wave, with magnitude=1
                rlIndF=find(rlInd);
                for pWaveInd=1:sum(rlInd)
                    Xhulm(n).bhlm{pWaveInd}=[rlAll(symInd).rlnl1(rlIndF(pWaveInd),[1:2]) 1];
                end
                
                  
            % Set q
            Xhulm(n).q=q(qInd);
            
            % Set symm label
            Xhulm(n).irep=rlAll(symInd).symm;
            
            % Set no. of elements h
            Xhulm(n).h=length(Xhulm(n).bhlm);
            
            %*** Aditional indexing, useful for fitting etc.
            % Index unique (l,m) pairs for each subset
            [Xhulm(n).lmPairs,Xhulm(n).lmPairsRowInd,Xhulm(n).lmPairsRowInd2]=unique(rlAll(symInd).rlnl1(rlInd,1:2),'rows','stable');
            
            %*** Check if conjugates are reqd. for selected mat. elements
            % (1) Degenerate states, single q - in this case it's a frame rotation and real & imag parts are swapped
            degenInd=[];
            degen=unique(rlAll(symInd).rlnl1(rlInd,6),'rows','stable');
            for degenState=1:length(degen)
                degenInd(degenState,:)=find(rlAll(symInd).rlnl1(rlInd,6)==degen(degenState)); % Check degeneracies and index pairs in selected mat. elements
                % find(rlAll(symInd).rlnl1(rlInd,6)==degen(degenState))
            end
            Xhulm(n).degenInd=degenInd;
            Xhulm(n).degen=degen;
            
            rotInd=zeros(size(degenInd));
            if size(degenInd,1)>1           % Only run this code if there are degenerate states!
                if abs(real(Xhulm(n).dlnl(degenInd(1,1),5)))==abs(imag(Xhulm(n).dlnl(degenInd(2,1),5)))
                    rotInd(2,:)=1;
                end
            end
                
            Xhulm(n).rotInd=rotInd;
            
            % (2) q=+/-1 states, in this case check if mat. elements are identical or if it's a conjugate relation
            conjInd=0;
            repInd=0;
            if q(qInd)==1
                if (n-1)>0
                    if (Xhulm(n-1).q==-1)&&(strcmp(Xhulm(n-1).irep,Xhulm(n).irep))
                        repInd=find(Xhulm(n-1).dlnl(:,5)==Xhulm(n).dlnl(:,5));
                        conjInd=find((real(Xhulm(n-1).dlnl(:,5))==-real(Xhulm(n).dlnl(:,5)))&(imag(Xhulm(n-1).dlnl(:,5))==-imag(Xhulm(n).dlnl(:,5))));
                    end
                end
            end
            Xhulm(n).repInd=repInd; 
            Xhulm(n).conjInd=conjInd;
                
            % (3) m=+/- states, may be identical or conjugates depending on symmetry
            mRepInd=0;
            mConjInd=0;
            
            % For each l, check m pairs
            for m=2:length(Xhulm(n).lmPairsRowInd)
                mInd=Xhulm(n).lmPairsRowInd(m);
                if (mInd-1)>0
                    % Xhulm(n).rlAllIn(mInd,1)
                    % Check that given lm pair and previous correspond to l, +/-m state
                    if (Xhulm(n).rlAllIn(mInd,1)==Xhulm(n).rlAllIn(mInd-1,1))&&(Xhulm(n).rlAllIn(mInd,2)==(-Xhulm(n).rlAllIn(mInd-1,2)))  
                        
                        % Index identical and conjugate states - may need to change to Hermitian conjugate defn. above.
                        mRepInd(mInd)=Xhulm(n).dlnl(mInd,5)==Xhulm(n).dlnl(mInd-1,5);
                        
                    end
                end
            end
            
            % If there are +/-m pairs present, check for conjugate relations
            if sum(mRepInd)
                mConjInd=((real(Xhulm(n).dlnl(1:end-1,5))==-real(Xhulm(n).dlnl(2:end,5)))&(imag(Xhulm(n).dlnl(1:end-1,5))==-imag(Xhulm(n).dlnl(2:end,5))));
                mConjInd=[0; mConjInd];  % Correct dimension
            end
            
            Xhulm(n).mRepInd=mRepInd; 
            Xhulm(n).mConjInd=mConjInd;
                        
            n=n+1;
        end
    
    else
        Xhulm(n).dlnl(1,:)=[1 0 0];
        Xhulm(n).bhlm{1}=[0 0 0];
        Xhulm(n).q=[];
        Xhulm(n).irep=rlAll(symInd).symm;
        Xhulm(n).h=0;
        
        n=n+1;
    end

end

if Xflag
    %% *** Calculate harmonics from bplm and plot
    res=100;
    [theta,phi]=meshgrid(linspace(0,pi,res),linspace(0,2*pi,res));
    
    for n=1:length(Xhulm)
        for h=1:Xhulm(n).h;   % Loop over elements
            % Xhulm(n).Ihulm{h}=pad_sph_2D([Xhulm(n).bhlm{h} 0*ones(size(Xhulm(n).bhlm{h},1),1)], 50); % Calculate & plot
            Xhulm(n).Ihulm{h}=pad_sph_2D([Xhulm(n).bhlm{h} 0*ones(size(Xhulm(n).bhlm{h},1),1)], theta,phi); % Calculate & plot
        end
    end
end